import { client } from "../../../index.js";
import { AGame } from "../AGame.js";

class TicTacToe
{
    constructor(height, width, gap, rectsize, canvas, game_id)
    {
        this.height = height;
        this.width = width;
        this.gap = gap;
        this.rectsize = rectsize;
        this.map = [[],[],[],[],[],[],[],[],[]];
        for (let i = 0; i < 9; i++)
			for (let j = 0; j < 9; j++)
				this.map[i].push(-1);
		this.game_id = game_id;
		this.game = new AGame(client, game_id, this.onReceive.bind(this), this.uninit.bind(this), "tictactoe")
        this.canvas = canvas
        this.context = this.canvas.getContext("2d");
		this.sign = "x";
		this.turn;
    }

	async init()
	{
		console.log(this.game_id);
		await this.game.join();
		this.canvas.addEventListener("mousedown", (event, morpion = this) => this.onClick(event, morpion));
	}

	async uninit()
	{
		this.canvas.removeEventListener("mousedown", (event, morpion = this) => this.onClick(event, morpion));
	}

	async onReceive(messageData)
	{
		console.log(messageData)
		if (messageData.detail == "x" || messageData.detail == "o")
		{
			this.sign = messageData.detail;
			this.turn = messageData.detail == "x";
		}
		else if (messageData.detail == "game_start")
			this.game.started = true;
		else
		{
			this.map[messageData.targetMorpion][messageData.targetCase] = (this.sign == "x") ? 1 : 0;
			this.printSign(messageData.targetMorpion, messageData.targetCase, (this.sign == "x") ? "o" : "x")
		}
	}

	onClick(event, morpion)
	{
		let x = event.offsetX;
		let y = event.offsetY;
		let targetMorpion, targetCase;

		targetMorpion = morpion.findPlace(x, this) + morpion.findPlace(y, this) * 3;
		if (morpion.findPlace(x, this) < 0 || morpion.findPlace(y, this) < 0)
			return -1;
		targetCase = morpion.findSquare(x, this.rectsize * 3 * morpion.findPlace(x, this) + this.gap, this) + morpion.findSquare(y, this.rectsize * 3 * morpion. findPlace(y, this) + this.gap, this) * 3;

		if (morpion.checkCase(targetMorpion, targetCase))
		{
			morpion.sendCase(targetMorpion, targetCase);
			morpion.setOutline();
		}
		else
			morpion.incorrectCase();
	}

	checkCase(targetMorpion, targetCase)
	{
		console.log("TargetMorpion :" + targetMorpion + " targetCase :" + targetCase);
		return (this.map[targetMorpion][targetCase] == -1 && this.turn == true);
	}

	incorrectCase()
	{
		console.log("bozo");
	}

	sendCase(targetMorpion, targetCase)
	{
		this.map[targetMorpion][targetCase] = (this.sign == "x") ? 0 : 1; // soit pas un bozo relis
		this.printSign(targetMorpion, targetCase, this.sign);
		console.log(this.game.send, targetMorpion, targetCase)
		this.game.send(JSON.stringify({"targetMorpion" : targetMorpion, "targetCase" : targetCase}));
		console.log(this.turn);
		this.turn = !this.turn;
	}

	printSign(targetMorpion, targetCase, sign)
	{
		let targetX = (this.gap + targetMorpion % 3 * this.rectsize * 3) + (targetCase % 3 * this.rectsize);
		let targetY = (this.gap + Math.floor(targetMorpion / 3) * this.rectsize * 3) + (Math.floor(targetCase / 3) * this.rectsize);
		if (sign == "x")
		{
			this.context.beginPath();
			this.context.strokeStyle = "green";
			this.context.moveTo(targetX + 15, targetY + 15);
			this.context.lineTo(targetX + 45, targetY + 45);
			this.context.moveTo(targetX + 45, targetY + 15);
			this.context.lineTo(targetX + 15, targetY + 45);
			this.context.stroke();
			this.context.closePath();
		}
		else
		{
			this.context.beginPath();
			this.context.strokeStyle = "red";
			targetX += this.rectsize / 2;
			targetY += this.rectsize / 2;
			this.context.arc(targetX, targetY, 20, 0, 2 * Math.PI);
			this.context.stroke();
			this.context.closePath();
		}
		if (sign != this.sign)
			this.turn = true;
	}

	findPlace(x, morpion)
	{	
		if (x <= this.gap || x >= this.gap + this.rectsize * 9)
			return -1;
		if (x <= this.gap + this.rectsize * 3)
			return 0;
		if (x >= this.gap + this.rectsize * 3 && x <= this.gap + this.rectsize * 6)
			return 1;
		if (x >= this.gap + this.rectsize * 6)
			return 2;
		return -1;
	}

	findSquare(x, gap, morpion)
	{
		if (x <= gap + this.rectsize)
			return 0;
		if (x >= gap + this.rectsize && x <= gap + this.rectsize * 2)
			return 1;
		if (x >= gap + this.rectsize * 2)
			return 2;
		return -1;
	}

	setOutline()
	{
		if (this.turn)
		{
			this.context.beginPath();
			this.context.strokeStyle = (this.sign == "x") ? "green" : "red";
			this.context.roundRect(0, 0, this.canvas.width, this.canvas.height, 25);
			this.context.stroke();
			this.context.closePath();
		}
		else
		{
			this.context.beginPath();
			this.context.strokeStyle = "#1a1a1d";
			this.context.roundRect(0, 0, this.canvas.width, this.canvas.height, 25);
			this.context.stroke();
			this.context.closePath();
		}
	}

    DrawSuperMorpion()
    {
        this.context.fillStyle = "#1a1a1d";
		this.context.roundRect(0, 0, this.canvas.width, this.canvas.height, 20);
        this.context.fill();
		for (let i = 1, x = this.gap, y  = this.gap; i <= 9; i++)
		{
			this.DrawMorpion(x, y);
			x += this.rectsize * 3;
			if (i % 3 == 0)
			{
				y += this.rectsize * 3;
				x = this.gap;
			}
		}
		this.context.lineWidth = 6;
		for (let i = 0; i < 4; i++)
		{
			this.context.beginPath();
			this.context.strokeStyle = `rgb(230 230 230)`;
			this.context.moveTo(this.gap + i * this.rectsize * 3, this.gap - 3);
			this.context.lineTo(this.gap + i * this.rectsize * 3, this.canvas.height - this.gap + 3);
			this.context.stroke();
			this.context.closePath();
		};
		for (let i = 0; i < 4; i++)
		{
			this.context.beginPath();
			this.context.strokeStyle = `rgb(230 230 230)`;
			this.context.moveTo(this.gap, this.gap + i * this.rectsize * 3);
			this.context.lineTo(this.canvas.height - this.gap, this.gap + i * this.rectsize * 3);
			this.context.stroke();
			this.context.closePath();
		}
    }

    DrawMorpion(start_x, start_y)
    {
        this.context.beginPath();
		this.context.strokeStyle = `rgb(200 200 200)`;
		for (let i = 1, x = 0, y = 0; i <= 9; i++)
		{
			this.context.strokeRect(start_x + x, start_y + y, this.rectsize, this.rectsize);
			x += this.rectsize;
			if (i % 3 == 0)
			{
				y += this.rectsize;
				x = 0;
			}
		}
		this.context.closePath();
    }

	selectCase(x, y)
	{
		case_morpion = Math.floor(x / this.rectsize) + Math.floor((y / this.rectsize)) * 3
		case_square = Math.floor(x / Math.floor(this.rectsize / 3)) % this.rectsize + Math.floor(y / this.rectsize) % this.rectsize
		// ask server if case_morpion == playing_case && case_square == empty

	}

	setOutline()
	{
		if (this.turn)
		{
			this.context.beginPath();
			this.context.strokeStyle = (this.sign == "x") ? "green" : "red";
			this.context.roundRect(0, 0, this.canvas.width, this.canvas.height, 25);
			this.context.stroke();
			this.context.closePath();
		}
		else
		{
			this.context.beginPath();
			this.context.strokeStyle = "#1a1a1d";
			this.context.roundRect(0, 0, this.canvas.width, this.canvas.height, 25);
			this.context.stroke();
			this.context.closePath();
		}
	}
}

export { TicTacToe };